﻿using System;
using System.Globalization;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using C1.Win.C1SpellChecker;

namespace C1DBSpellChecker
{
    public partial class SpellCheckerOptionsDialog : Form
    {
        C1SpellChecker _spell;

        public SpellCheckerOptionsDialog()
        {
            InitializeComponent();

            // populate main dictionary combobox
            _cmbMainDict.Items.Clear();
            _cmbMainDict.Items.Add("English US (default)");
            foreach (string f in Directory.GetFiles(Application.StartupPath, "*.dct"))
            {
                var fileName = Path.GetFileName(f);
                if (fileName.StartsWith("C1Spell_", StringComparison.OrdinalIgnoreCase))
                {
                    var item = new MainDictionaryWrapper(fileName);
                    _cmbMainDict.Items.Add(item);
                }
            }
        }

        // make user dictionary button and text box the same height
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            _btnUserDict.Height = _tbUserDict.Height;
        }

        // get or set the C1SpellChecker we're configuring
        public C1SpellChecker SpellChecker
        {
            get { return _spell; }
            set 
            {
                _spell = value;
                LoadOptions();
            }
        }

        // apply options to the C1SpellChecker
        void _btnOK_Click(object sender, EventArgs e)
        {
            SaveOptions();
        }

        // select a user dictionary
        void _btnUserDict_Click(object sender, EventArgs e)
        {
            using (var dlg = new SaveFileDialog())
            {
                dlg.DefaultExt = ".dct";
                if (dlg.ShowDialog() == DialogResult.OK)
                {
                    _tbUserDict.Text = dlg.FileName;
                }
            }
        }

        // initialize dialog settings from C1SpellChecker
        void LoadOptions()
        {
            // get ignore options
            var i = _spell.Options.Ignore;
            _chkHtml.Checked = (i & IgnoreOptions.HtmlTags) != 0;
            _chkMixed.Checked = (i & IgnoreOptions.MixedCase) != 0;
            _chkNumbers.Checked = (i & IgnoreOptions.Numbers) != 0;
            _chkUpper.Checked = (i & IgnoreOptions.UpperCase) != 0;
            _chkUrls.Checked = (i & IgnoreOptions.Urls) != 0;

            // get main dictionary
            var fn = _spell.MainDictionary.FileName;
            _cmbMainDict.SelectedIndex = 0;
            foreach (object item in _cmbMainDict.Items)
            {
                var wrapper = item as MainDictionaryWrapper;
                if (wrapper != null && wrapper.FileName == fn)
                {
                    _cmbMainDict.SelectedItem = item;
                    break;
                }
            }

            // get user dictionary
            _tbUserDict.Text = _spell.UserDictionary.FileName;
        }

        // apply dialog settings to C1SpellChecker 
        void SaveOptions()
        {
            // save ignore options
            var i = IgnoreOptions.None;
            if (_chkHtml.Checked) i |= IgnoreOptions.HtmlTags;
            if (_chkMixed.Checked) i |= IgnoreOptions.MixedCase;
            if (_chkNumbers.Checked) i |= IgnoreOptions.Numbers;
            if (_chkUpper.Checked) i |= IgnoreOptions.UpperCase;
            if (_chkUrls.Checked) i |= IgnoreOptions.Urls;
            _spell.Options.Ignore = i;

            // save main dictionary
            _spell.MainDictionary.FileName = _cmbMainDict.SelectedIndex > 0
                ? ((MainDictionaryWrapper)_cmbMainDict.SelectedItem).FileName
                : string.Empty;

            // save user dictionary
            _spell.UserDictionary.FileName = _tbUserDict.Text;
        }

        // wrapper class to map dictionary file names and cultures
        class MainDictionaryWrapper
        {
            string _fileName;
            CultureInfo _ci;
            public MainDictionaryWrapper(string fileName)
            {
                _fileName = Path.GetFileName(fileName);
                var cultureName = Path.GetFileNameWithoutExtension(fileName).Substring(8);
                if (cultureName.Length == 5 && cultureName[2] == '-')
                {
                    try
                    {
                        _ci = new CultureInfo(cultureName);
                    }
                    catch { }
                }
            }
            public string FileName
            {
                get { return _fileName; }
            }
            public override string ToString()
            {
                return _ci != null ? _ci.DisplayName : _fileName;
            }
        }
    }
}
